' Licensed Materials - Property of IBM
'
' (C) Copyright IBM Corp. 1999, 2010 All Rights Reserved
'
' US Government Users Restricted Rights - Use, duplicate or
' disclosure restricted by GSA ADP Schedule Contract with
' IBM Corp.

'==============================
' Initialize
'==============================
  Set wbemSvc = GetObject("winmgmts://" & "." & "/root/cimv2")
  Set fileSys = CreateObject("Scripting.FileSystemObject")
  windowsDir = FindWindowsDir
 
'==============================
' Main(Args[])
'==============================

  EnumerateDeviceDrivers

'==============================
' EnumerateDeviceDrivers
'==============================
Function EnumerateDeviceDrivers()

  Set driverSet = wbemSvc.ExecQuery("Select * from Win32_SystemDriver")

  index = 0
  For Each driver In driverSet
    PrintDriverInfo index,driver
    index = index + 1
  Next

  FindSpecialDrivers index

End Function

'==============================
' FindSpecialDrivers
'==============================
Function FindSpecialDrivers(index)

  index = FindSpecialDriver(index,"ibmspwdll","ibmspw.dll","rsa2","RSA II Daemon")
  index = FindSpecialDriver(index,"ibmsp6wdll","ibmsp6w.dll","ibmipmi","Mapping Layer for OSA IPMI")
  index = FindSpecialDriver(index,"ibmspw64dll","ibmspw64.dll","rsa2","RSA II Daemon - 64 bit")
  index = FindSpecialDriver(index,"ibmsp6w64dll","ibmsp6w64.dll","ibmipmi","Mapping Layer for OSA IPMI - 64 bit")
  index = FindSpecialDriver(index,"ibmsprdpexe","ibmsprdp.exe","null","RSA II Terminal Services Bridge")

End Function

'==============================
' FindSpecialDriver
'==============================
Function FindSpecialDriver(index, driverName, driverFile, subDir, description)

  FindSpecialDriver=index
' ### Search for the Driver in the following location order: ###
' ## C:\Windows\system32 
  pathName = windowsDir&"\system32\"&driverFile
  If fileSys.FileExists(pathName) = false Then
'   ## C:\Windows\system32\drivers
    pathName = windowsDir&"\system32\drivers\"&driverFile
    If fileSys.FileExists(pathName) = false Then
'     ## C:\Windows\system32\drivers\subdir
      pathName = windowsDir&"\system32\drivers\"&subdir&"\"&driverFile
      If fileSys.FileExists(pathName) = false Then
'       ## C:\Windows\syswow64
        pathName = windowsDir&"\syswow64\"&driverFile
        If fileSys.FileExists(pathName) = false Then
'         ## C:\Windows\syswow64\drivers
          pathName = windowsDir&"\syswow64\drivers\"&driverFile
          If fileSys.FileExists(pathName) = false Then
'           ## C:\Windows\syswow64\drivers\subdir
            pathName = windowsDir&"\syswow64\drivers\"&subdir&"\"&driverFile
          End If
        End If
      End If
    End If
  End If

  If fileSys.FileExists(pathName) Then
    PrintSpecialDriverInfo index,driverName,pathName,description
    FindSpecialDriver=index+1
  End If

End Function

'==============================
' PrintDriverInfo
'==============================
Function PrintDriverInfo(index, driver)

  WScript.StdOut.WriteLine "DeviceDriver." & index & ".Name=" & driver.Name
  WScript.StdOut.WriteLine "DeviceDriver." & index & ".Description=" & driver.Description
  WScript.StdOut.WriteLine "DeviceDriver." & index & ".PathName=" & driver.PathName
  WScript.StdOut.WriteLine "DeviceDriver." & index & ".ServiceType=" & driver.ServiceType
  WScript.StdOut.WriteLine "DeviceDriver." & index & ".StartMode=" & driver.StartMode
  WScript.StdOut.WriteLine "DeviceDriver." & index & ".Started=" & driver.Started

  PrintDriverFileInfo index,driver.PathName

End Function

'==============================
' PrintSpecialDriverInfo
'==============================
Function PrintSpecialDriverInfo(index, driverName, pathName, description)

  WScript.StdOut.WriteLine "DeviceDriver." & index & ".Name=" & driverName
  WScript.StdOut.WriteLine "DeviceDriver." & index & ".Description=" & description
  WScript.StdOut.WriteLine "DeviceDriver." & index & ".PathName=" & pathName
  WScript.StdOut.WriteLine "DeviceDriver." & index & ".ServiceType=Kernel Driver"
  WScript.StdOut.WriteLine "DeviceDriver." & index & ".StartMode=System"
  WScript.StdOut.WriteLine "DeviceDriver." & index & ".Started=True"

  PrintDriverFileInfo index,pathName

End Function

'==============================
' GetProductVersion
'  This is a different way of obtaining version information
'  We use it to get driver info for drivers like Active PCI
'==============================
Function GetProductVersion (fileObj)
Dim objShell, objFolder, objFolderItem, i 
    Set objShell = CreateObject("Shell.Application")
    Set objFolder = objShell.NameSpace(fileSys.GetParentFolderName(fileObj))
    'Set objFolder = fileObj.ParentFolder
    Set objFolderItem = objFolder.ParseName(fileObj.Name)
    Dim arrHeaders(300)
    For i = 0 To 300
        arrHeaders(i) = objFolder.GetDetailsOf(objFolder.Items, i)
        'WScript.Echo i &"- " & arrHeaders(i) & ": " & objFolder.GetDetailsOf(objFolderItem, i)
        If lcase(arrHeaders(i))= "product version" Then
            GetProductVersion= objFolder.GetDetailsOf(objFolderItem, i)
            Exit For
        End If
    Next
End Function

'==============================
' PrintDriverFileInfo
'==============================
Function PrintDriverFileInfo(index, pathName)
  
  pathName = FormatPathName(pathName)

  If Len(pathName) > 0 Then
    If fileSys.FileExists(pathName) Then
      Set fileObj = fileSys.GetFile(pathName)
      WScript.StdOut.WriteLine "DeviceDriver." & index & ".FileName=" & fileObj.Name
      WScript.StdOut.WriteLine "DeviceDriver." & index & ".FileType=" & fileObj.Type
      WScript.StdOut.WriteLine "DeviceDriver." & index & ".FileSize=" & fileObj.Size
      WScript.StdOut.WriteLine "DeviceDriver." & index & ".LastModified=" & fileObj.DateLastModified
    If fileObj.Name = "bfadfcoe.sys" OR fileObj.Name = "bfad.sys" OR _
	   fileObj.Name = "bnad5_32.sys" OR fileObj.Name = "bnad5_64.sys" OR _
	   fileObj.Name = "bnad5.sys" OR fileObj.Name = "bnad6.sys" Then
		HandleBrocadeVersion index, fileObj, fileSys, pathName
    ElseIf fileObj.Name = "ibmhpa.sys" Then
           WScript.StdOut.WriteLine "DeviceDriver." & index & ".Version=" & GetProductVersion(fileObj)
    ElseIf fileObj.Name = "nfrd960.sys" Then
           WScript.StdOut.WriteLine "DeviceDriver." & index & ".Version=" & GetProductVersion(fileObj)
    Else
           WScript.StdOut.WriteLine "DeviceDriver." & index & ".Version=" & fileSys.GetFileVersion(pathName)
    End If
    WScript.StdOut.WriteLine "DeviceDriver." & index & ".SoftwareID=" & fileObj.Name

    End If
  End If 

End Function

'==============================
' FindWindowsDir
'==============================
Function FindWindowsDir()

  FindWindowsDir = "C:\WINDOWS"
  Set osSet = wbemSvc.ExecQuery("Select * from Win32_OperatingSystem")

  For Each osInst In osSet
    FindWindowsDir = osInst.WindowsDirectory
  Next

End Function

'==============================
' FormatPathName
'==============================
Function FormatPathName(pathName)

  If Len(pathName) > 0 Then
    pathName = Replace(pathName,"\??\","")
  End If 

  FormatPathName = pathName

End Function

'==============================
' HandleBrocadeVersion
'==============================
Function HandleBrocadeVersion(index, fileObj, fileSys, pathName)


	found = false

	if left(fileSys.GetFileVersion(pathName),1) = "6" Then
		' This is old stuff ... pre 2.3.0.0
		

		Set regExF = New RegExp 	
		regExF.Pattern = "F"
		regExF.IgnoreCase = false	

		Set regExil = New RegExp 	
		regExil.Pattern = "i.l.e.V.e.r.s.i.o.n"
		regExil.IgnoreCase = false

		Set MyFile = fileSys.OpenTextFile(pathName, 1)
		Do While MyFile.AtEndOfStream <> True
			TextLine = MyFile.Read(2)
			retVal = regExF.Test(TextLine)
			If retVal Then
				'TextLine = MyFile.Read(1)
   				TextLine = MyFile.Read(20)

				retVal = regExil.Test(TextLine)
				If retVal Then
					Set regExNum = New RegExp
					regExNum.Pattern = "\d{1}.\..\d{1}.\..\d{1}.\..\d{1}"
					TextLine = MyFile.Read(20)
					Set myVersion = regExNum.Execute(TextLine)
					FinalVersion = Left(myVersion.Item(0),1) + "." + Mid(myVersion.Item(0),5,1) + "." + Mid(myVersion.Item(0),9,1) + "." + Right(myVersion.Item(0),1)
					WScript.StdOut.WriteLine "DeviceDriver." & index & ".Version=" & FinalVersion	
					found = true
				End If
			End If
		Loop

		MyFile.Close   


	End If

	If found = false Then

		WScript.StdOut.WriteLine "DeviceDriver." & index & ".Version=" & fileSys.GetFileVersion(pathName)      

	End If



End Function

